"""
Extends the AlarmDecoder class to include helper methods for HomeAssistant
"""

import asyncio

from alarmdecoder.decoder import AlarmDecoder
from alarmdecoder.panels import DSC, ADEMCO as HONEYWELL

ARM_HOME = "arm_home"
ARM_AWAY = "arm_away"
ARM_NIGHT = "arm_night"


class AdExt(AlarmDecoder):
    """
    Extended AlarmDecoder class
    """

    def __init__(self, device, ignore_message_states=False, ignore_lrr_states=True):
        super().__init__(device, ignore_message_states, ignore_lrr_states)
        self._event = asyncio.Event()

    def _handle_version(self, data):
        super()._handle_version(data)
        self._event.set()

    async def is_init(self):
        return (await self._event.wait())

    def _get_arm_sequence(
        self, arm_mode, code, code_arm_required, auto_bypass, alt_night_mode=False,
    ):
        """
        Returns arming sequences based on panel brand & config settings
        """

        arming_sequences = {
            HONEYWELL: {
                "code": {
                    ARM_HOME: f"{code!s}3",
                    ARM_AWAY: f"{code!s}2",
                    ARM_NIGHT: f"{code!s}33" if alt_night_mode else f"{code!s}7",
                },
                "nocode": {
                    ARM_HOME: "#3",
                    ARM_AWAY: "#2",
                    ARM_NIGHT: f"{code!s}33" if alt_night_mode else "#7",
                },
            },
            DSC: {
                "code": {
                    ARM_HOME: str(code),
                    ARM_AWAY: str(code),
                    ARM_NIGHT: f"*9{code!s}" if alt_night_mode else str(code),
                },
                "nocode": {
                    ARM_HOME: chr(4) * 3,
                    ARM_AWAY: chr(5) * 3,
                    ARM_NIGHT: f"*9{code!s}" if alt_night_mode else chr(4) * 3,
                },
            },
        }

        brand = self.mode
        if code_arm_required and not code:
            return ""
        if alt_night_mode and arm_mode == ARM_NIGHT and not code:
            return ""

        auto_bypass_code = (
            f"{code!s}6#" if auto_bypass and brand == HONEYWELL and code else ""
        )
        code_key = "code" if code_arm_required else "nocode"
        return auto_bypass_code + arming_sequences[brand][code_key][arm_mode]

    def _arm_panel(self, arm_mode, **kwargs):
        arm_sequence = self._get_arm_sequence(arm_mode, **kwargs)
        if arm_sequence:
            return self.send(arm_sequence)

    def arm_night(
        self, code=None, code_arm_required=True, alt_night_mode=False, auto_bypass=False
    ):
        """Arms an alarm panel in night mode"""
        arm_mode = ARM_NIGHT
        return self._arm_panel(
            arm_mode,
            code=code,
            code_arm_required=code_arm_required,
            alt_night_mode=alt_night_mode,
            auto_bypass=auto_bypass,
        )

    def arm_home(self, code=None, code_arm_required=True, auto_bypass=False):
        """Arms an alarm panel in home/stay mode"""
        arm_mode = ARM_HOME
        return self._arm_panel(
            arm_mode,
            code=code,
            code_arm_required=code_arm_required,
            auto_bypass=auto_bypass,
        )

    def arm_away(self, code=None, code_arm_required=True, auto_bypass=False):
        """Arms an alarm panel in away mode"""
        arm_mode = ARM_AWAY
        return self._arm_panel(
            arm_mode,
            code=code,
            code_arm_required=code_arm_required,
            auto_bypass=auto_bypass,
        )



"""
Setup.py for adext
"""
from os import path
from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="adext",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    description="AlarmDecoder extended",
    url="https://github.com/ajschmidt8/adext",
    author="AJ Schmidt",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    packages=["adext"],
    install_requires=["alarmdecoder==1.13.11"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications",
        "Topic :: Home Automation",
        "Topic :: Security",
    ],
)



"""Tests for adext"""

from unittest.mock import Mock
import pytest
from alarmdecoder.panels import DSC, ADEMCO as HONEYWELL
from adext import AdExt


@pytest.mark.parametrize(
    "code,code_arm_required,auto_bypass,brand,arm_sequence",
    #    code  req   bypa
    [
        (1234, True, True, HONEYWELL, "12346#12343"),
        (1234, True, False, HONEYWELL, "12343"),
        (1234, False, True, HONEYWELL, "12346##3"),
        (1234, False, False, HONEYWELL, "#3"),
        (None, True, True, HONEYWELL, ""),
        (None, True, False, HONEYWELL, ""),
        (None, False, True, HONEYWELL, "#3"),
        (None, False, False, HONEYWELL, "#3"),
        (1234, True, True, DSC, "1234"),
        (1234, True, False, DSC, "1234"),
        (1234, False, True, DSC, chr(4) * 3),
        (1234, False, False, DSC, chr(4) * 3),
        (None, True, True, DSC, ""),
        (None, True, False, DSC, ""),
        (None, False, True, DSC, chr(4) * 3),
        (None, False, False, DSC, chr(4) * 3),
    ],
)
def test_arm_home(code, code_arm_required, auto_bypass, brand, arm_sequence):
    """Test all variations of arm_home"""
    ad = AdExt("device")
    ad.mode = brand
    ad.send = Mock()
    ad.arm_home(
        code=code, code_arm_required=code_arm_required, auto_bypass=auto_bypass,
    )

    if arm_sequence:
        return ad.send.assert_called_once_with(arm_sequence)
    return ad.send.assert_not_called()


@pytest.mark.parametrize(
    "code,code_arm_required,auto_bypass,brand,arm_sequence",
    #    code  req   bypa
    [
        (1234, True, True, HONEYWELL, "12346#12342"),
        (1234, True, False, HONEYWELL, "12342"),
        (1234, False, True, HONEYWELL, "12346##2"),
        (1234, False, False, HONEYWELL, "#2"),
        (None, True, True, HONEYWELL, ""),
        (None, True, False, HONEYWELL, ""),
        (None, False, True, HONEYWELL, "#2"),
        (None, False, False, HONEYWELL, "#2"),
        (1234, True, True, DSC, "1234"),
        (1234, True, False, DSC, "1234"),
        (1234, False, True, DSC, chr(5) * 3),
        (1234, False, False, DSC, chr(5) * 3),
        (None, True, True, DSC, ""),
        (None, True, False, DSC, ""),
        (None, False, True, DSC, chr(5) * 3),
        (None, False, False, DSC, chr(5) * 3),
    ],
)
def test_arm_away(code, code_arm_required, auto_bypass, brand, arm_sequence):
    """Test all variations of arm_away"""
    ad = AdExt("device")
    ad.mode = brand
    ad.send = Mock()
    ad.arm_away(
        code=code, code_arm_required=code_arm_required, auto_bypass=auto_bypass,
    )

    if arm_sequence:
        return ad.send.assert_called_once_with(arm_sequence)
    return ad.send.assert_not_called()


@pytest.mark.parametrize(
    "code,code_arm_required,alt_night_mode,auto_bypass,brand,arm_sequence",
    #    code  req   alt   bypa
    [
        (1234, True, True, True, HONEYWELL, "12346#123433"),
        (1234, True, True, False, HONEYWELL, "123433"),
        (1234, True, False, True, HONEYWELL, "12346#12347"),
        (1234, True, False, False, HONEYWELL, "12347"),
        (1234, False, True, True, HONEYWELL, "12346#123433"),
        (1234, False, True, False, HONEYWELL, "123433"),
        (1234, False, False, True, HONEYWELL, "12346##7"),
        (1234, False, False, False, HONEYWELL, "#7"),
        (None, True, True, True, HONEYWELL, ""),
        (None, True, True, False, HONEYWELL, ""),
        (None, True, False, True, HONEYWELL, ""),
        (None, True, False, False, HONEYWELL, ""),
        (None, False, True, True, HONEYWELL, ""),
        (None, False, True, False, HONEYWELL, ""),
        (None, False, False, True, HONEYWELL, "#7"),
        (None, False, False, False, HONEYWELL, "#7"),
        (1234, True, True, True, DSC, "*91234"),
        (1234, True, True, False, DSC, "*91234"),
        (1234, True, False, True, DSC, "1234"),
        (1234, True, False, False, DSC, "1234"),
        (1234, False, True, True, DSC, "*91234"),
        (1234, False, True, False, DSC, "*91234"),
        (1234, False, False, True, DSC, chr(4) * 3),
        (1234, False, False, False, DSC, chr(4) * 3),
        (None, True, True, True, DSC, ""),
        (None, True, True, False, DSC, ""),
        (None, True, False, True, DSC, ""),
        (None, True, False, False, DSC, ""),
        (None, False, True, True, DSC, ""),
        (None, False, True, False, DSC, ""),
        (None, False, False, True, DSC, chr(4) * 3),
        (None, False, False, False, DSC, chr(4) * 3),
    ],
)
def test_arm_night(
    code, code_arm_required, alt_night_mode, auto_bypass, brand, arm_sequence
):
    """Test all variations of arm_night."""
    ad = AdExt("device")
    ad.mode = brand
    ad.send = Mock()
    ad.arm_night(
        code=code,
        code_arm_required=code_arm_required,
        auto_bypass=auto_bypass,
        alt_night_mode=alt_night_mode,
    )

    if arm_sequence:
        return ad.send.assert_called_once_with(arm_sequence)
    return ad.send.assert_not_called()


def test_arm_night_no_alt():
    """Test arm_night without alt_night_mode arg"""
    ad = AdExt("device")
    ad.mode = HONEYWELL
    ad.send = Mock()
    ad.arm_night(
        code="1234", code_arm_required=True, auto_bypass=True,
    )
    return ad.send.assert_called_once_with("12346#12347")
